@@ -1,7 +1,7 @@
 # cython.* namespace for pure mode.
 from __future__ import absolute_import
 
-__version__ = "0.29.20"
+__version__ = "0.25"
 
 try:
     from __builtin__ import basestring
@@ -108,14 +108,11 @@ class _Optimization(object):
 cclass = ccall = cfunc = _EmptyDecoratorAndManager()
 
 returns = wraparound = boundscheck = initializedcheck = nonecheck = \
-    embedsignature = cdivision = cdivision_warnings = \
-    always_allows_keywords = profile = linetrace = infer_types = \
+    overflowcheck = embedsignature = cdivision = cdivision_warnings = \
+    always_allows_keywords = profile = linetrace = infer_type = \
     unraisable_tracebacks = freelist = \
-        lambda _: _EmptyDecoratorAndManager()
+        lambda arg: _EmptyDecoratorAndManager()
 
-exceptval = lambda _=None, check=True: _EmptyDecoratorAndManager()
-
-overflowcheck = lambda _: _EmptyDecoratorAndManager()
 optimization = _Optimization()
 
 overflowcheck.fold = optimization.use_switch = \
@@ -123,8 +120,6 @@ overflowcheck.fold = optimization.use_switch = \
 
 final = internal = type_version_tag = no_gc_clear = no_gc = _empty_decorator
 
-binding = lambda _: _empty_decorator
-
 
 _cython_inline = None
 def inline(f, *args, **kwds):
@@ -149,7 +144,6 @@ def cdiv(a, b):
     q = a / b
     if q < 0:
         q += 1
-    return q
 
 def cmod(a, b):
     r = a % b
@@ -188,15 +182,8 @@ def declare(type=None, value=_Unspecified, **kwds):
         return value
 
 class _nogil(object):
-    """Support for 'with nogil' statement and @nogil decorator.
+    """Support for 'with nogil' statement
     """
-    def __call__(self, x):
-        if callable(x):
-            # Used as function decorator => return the function unchanged.
-            return x
-        # Used as conditional context manager or to create an "@nogil(True/False)" decorator => keep going.
-        return self
-
     def __enter__(self):
         pass
     def __exit__(self, exc_class, exc, tb):
@@ -206,7 +193,6 @@ nogil = _nogil()
 gil = _nogil()
 del _nogil
 
-
 # Emulated types
 
 class CythonMetaType(type):
@@ -396,7 +382,7 @@ py_complex = typedef(complex, "double complex")
 int_types = ['char', 'short', 'Py_UNICODE', 'int', 'Py_UCS4', 'long', 'longlong', 'Py_ssize_t', 'size_t']
 float_types = ['longdouble', 'double', 'float']
 complex_types = ['longdoublecomplex', 'doublecomplex', 'floatcomplex', 'complex']
-other_types = ['bint', 'void', 'Py_tss_t']
+other_types = ['bint', 'void']
 
 to_repr = {
     'longlong': 'long long',
@@ -431,17 +417,14 @@ for name in complex_types:
     gs[name] = typedef(py_complex, to_repr(name, name))
 
 bint = typedef(bool, "bint")
-void = typedef(None, "void")
-Py_tss_t = typedef(None, "Py_tss_t")
+void = typedef(int, "void")
 
 for t in int_types + float_types + complex_types + other_types:
     for i in range(1, 4):
-        gs["%s_%s" % ('p'*i, t)] = gs[t]._pointer(i)
+        gs["%s_%s" % ('p'*i, t)] = globals()[t]._pointer(i)
 
-NULL = gs['p_void'](0)
-
-# looks like 'gs' has some users out there by now...
-#del gs
+void = typedef(None, "void")
+NULL = p_void(0)
 
 integral = floating = numeric = _FusedType()
 
@@ -457,7 +440,7 @@ class CythonDotParallel(object):
     def parallel(self, num_threads=None):
         return nogil
 
-    def prange(self, start=0, stop=None, step=1, nogil=False, schedule=None, chunksize=None, num_threads=None):
+    def prange(self, start=0, stop=None, step=1, schedule=None, nogil=False):
         if stop is None:
             stop = start
             start = 0
